use image::{DynamicImage, RgbaImage};
use resvg::tiny_skia::{Paint, PathBuilder, Pixmap, PixmapPaint, Stroke};
use resvg::usvg::TreeParsing;
use resvg::usvg::{Options, Transform};
use resvg::Tree;

use crate::StdError;

// Render the raw SVG data to an image
pub fn render_svg(data: &[u8], height: u32, with_border: bool) -> StdError<DynamicImage> {
    let opt = Options::default();
    let rtree = resvg::usvg::Tree::from_data(data, &opt).expect("couldn't parse");
    let svg_size = rtree.size;
    // Work out how wide the pixmap of height `height` needs to be to entirely fit the SVG.
    let scale_factor = height as f32 / svg_size.height();
    let pm_width = (scale_factor * svg_size.width()).ceil() as u32;
    let mut pixmap = Pixmap::new(pm_width, height).ok_or("Error creating pixmap")?;
    // Render the svg into a pixmap.
    Tree::from_usvg(&rtree).render(
        Transform::from_scale(scale_factor, scale_factor),
        &mut pixmap.as_mut(),
    );
    // Make a wider pixmap with a 16:9 AR and the same height. This is a blesséd ratio by twitter
    // and means we see the whole image nicely in the timeline with no truncation.
    let mut bigger_pixmap =
        Pixmap::new(height / 9 * 16, height).ok_or("Error creating bigger pixmap")?;
    // Then draw our freshly rendered SVG into the middle of the bigger pixmap.
    bigger_pixmap.draw_pixmap(
        ((bigger_pixmap.width() - pm_width) / 2).try_into().unwrap(),
        0,
        pixmap.as_ref(),
        &PixmapPaint::default(),
        Transform::identity(),
        None,
    );
    let (w, h) = (bigger_pixmap.width(), bigger_pixmap.height());
    // Render a red border for debug purposes
    if with_border {
        let mut paint = Paint::default();
        paint.set_color_rgba8(255, 0, 0, 255);
        let stroke = Stroke {
            width: 1.0,
            ..Default::default()
        };

        let path = {
            let mut pb = PathBuilder::new();
            pb.move_to(0.0, 0.0);
            pb.line_to(0.0, h as f32 - stroke.width);
            pb.line_to(w as f32, h as f32 - stroke.width);
            pb.line_to(w as f32 - stroke.width, 0.0);
            pb.line_to(0.0, 0.0);
            pb.finish().unwrap()
        };
        bigger_pixmap.stroke_path(&path, &paint, &stroke, Transform::identity(), None);
    }
    let img = RgbaImage::from_raw(
        bigger_pixmap.width(),
        bigger_pixmap.height(),
        bigger_pixmap.data().to_vec(),
    )
    .ok_or("Error creating image from pixmap")?;
    Ok(DynamicImage::ImageRgba8(img))
}
